#
#	GTScriptLauncher for Maya
#	version 1.41,	21 June, 2014
#
#	created by Gotetz (http://gotetz-log.blogspot.jp/)
#

import sys
import os
import re
import codecs
import subprocess
from PySide import QtGui, QtCore
from maya.cmds import *
import maya.mel as mel
import maya.OpenMaya as OpenMaya
import maya.OpenMayaMPx as OpenMayaMPx

kPluginName = 'GTScriptLauncher';

class GTScriptLauncher_Pref:
	pref_dir = 'GTScriptLauncher_Path';
	pref_exe = 'GTScriptLauncher_Editor';
	windowName = "GTScriptLauncher_Dialog";
	fieldDir = '';
	fieldExe = '';
	
	def __init__(self):
		temp = optionVar(query=self.pref_dir);
		if type(temp) is int or not os.path.isdir(temp):
			GTScriptLauncher.scriptDir = GTScriptLauncher.defaultScriptDir;
		else:
			GTScriptLauncher.scriptDir = temp;
	
		temp = optionVar(query=self.pref_exe);
		if type(temp) == int or not os.path.exists(temp):
			GTScriptLauncher.editorPath = GTScriptLauncher.defaultEditorPath;
		else:
			GTScriptLauncher.editorPath = temp;
			
	def GetDirPath(self, *args):
		path = QtGui.QFileDialog.getExistingDirectory(None, 'Select Script Folder', GTScriptLauncher.scriptDir)
		path = re.sub(r'\\', r'\\\\', path);
		if '' != path:
			self.SetTextField(self.fieldDir, [path]);
		
	def GetExePath(self, *args):
		(path, type) = QtGui.QFileDialog.getOpenFileName(None, 'Select Text Editor', GTScriptLauncher.editorPath, '*.exe')
		path = re.sub(r'\\', r'\\\\', path);
		if '' != path:
			self.SetTextField(self.fieldExe, [path]);
			
	def SetTextField(self, ctrlName, path):
		path = os.path.normpath(path[0]);
		textField(ctrlName, edit=True, text=path);

	def DeleteDialog(self):
		if window(self.windowName, query=True, ex=True):
			deleteUI(self.windowName);	
			
	def ShowDialog(self, *args):
		self.DeleteDialog();
		window(self.windowName, title="GTScriptLauncher Settings", minimizeButton=False, maximizeButton=False, sizeable=False);
		columnLayout();
		rowLayout(numberOfColumns=3, columnWidth3=(70,360,30), columnAttach3=('right','both','both'),columnOffset3=(3,3,0), adjustableColumn=2);
		text('Directory')
		self.fieldDir = textField(text=GTScriptLauncher.scriptDir);
		button('...', command=self.GetDirPath);
		setParent('..');
		rowLayout(numberOfColumns=3, columnWidth3=(70,360,30), columnAttach3=('right','both','both'),columnOffset3=(3,3,0), adjustableColumn=2);
		text('TextEditor');
		self.fieldExe = textField(text=GTScriptLauncher.editorPath);
		button('...', command=self.GetExePath);
		setParent('..');
		rowLayout(numberOfColumns=2, columnAlign2=('left','left'), columnWidth2=(170,290));
		text('');
		button('Save && Close', width=120, command=self.SavePref);
		setParent('..');
		text(width=460, label='version 1.4.1', align='right')
		setParent('..');
		window(self.windowName, edit=True, width=460, height=90);
		showWindow(self.windowName);
                
	def SavePref(self, *args):
		GTScriptLauncher.scriptDir = os.path.normpath(textField(self.fieldDir, query=True, text=True));
		GTScriptLauncher.editorPath = os.path.normpath(textField(self.fieldExe, query=True, text=True));
		optionVar(stringValue=(self.pref_dir, GTScriptLauncher.scriptDir));
		optionVar(stringValue=(self.pref_exe, GTScriptLauncher.editorPath));
		self.DeleteDialog();
				
class GTScriptLauncher_Cmd:
	path = '';
	name = '';
	func = None;
	
	def __init__(self, type, path):
		self.path = os.path.normpath(path);
		self.name = os.path.basename(self.path);
		if type == '.mel':
			self.func = self.ExecMel;
		elif type == '.py':
			self.func = self.ExecPy;
		elif type == 'dir':
			self.func = self.Explorer;
	
	def ExecSwitcher(self):
		if 4 == getModifiers():
			exePath = os.path.normpath(GTScriptLauncher.editorPath);
			os.path.exists
			cmd = self.EncodeString(''.join(('"', exePath, ('" %s' % self.path))));
			subprocess.Popen(cmd, shell=True);
			return True;
		return False;
		
	def ExecMel(self, *args):
		if self.ExecSwitcher():
			return;
		self.path = re.sub(r'\\', r'\\\\', self.path);
		cmd = self.EncodeString('source "%s";' % self.path);
		mel.eval(cmd);
		GTScriptLauncher.lastExecuted = self;
		
	def ExecPy(self, *args):
		if self.ExecSwitcher():
			return;
		tmpDir = internalVar(userTmpDir=True)
		tmpFilePath = os.path.join(tmpDir, 'gtScriptLauncher_tempScript.py');

		with codecs.open(self.path, 'r', 'utf-8-sig', 'ignore') as fileIn:
			with codecs.open(tmpFilePath, 'w', 'shift-jis', 'ignore') as fileOut:
				for line in fileIn:
					fileOut.write(line)
		try:
			mel.eval("python(\"execfile(\'" + re.sub(r'\\', '/', tmpFilePath) + "\')\")")
		except:
			print '<GTScriptLauncher> Script execution error. \"%s\"' % self.path
			raise;
		aText = None;
		GTScriptLauncher.lastExecuted = self;
		
	def Explorer(self, *args):
		cmd = self.EncodeString('explorer %s' % self.path);
		subprocess.Popen(cmd, shell=True);

	def EncodeString(self, str):
		return str.encode('shift-jis');
		
class GTScriptLauncher:
	defaultScriptDir = os.path.normpath(internalVar(userScriptDir=True));
	scriptDir = defaultScriptDir;
	defaultEditorPath = 'notepad.exe';
	editorPath = defaultEditorPath;
	menuName = 'GTScriptLauncher';
	mayaWindow = mel.eval('$Temp=$gMainWindow');
	lastExecuted = None;
	
	def __init__(self):
		if self.ExistsMenu():
			deleteUI(self.menuName, menu=True);
			print "GTScriptLauncher Menu Rebuild"
		menu(self.menuName, parent=self.mayaWindow, tearOff=False, postMenuCommand=self.BuildMenu);
		
	def BuildMenu(self, *args):
		if self.ExistsMenu():
			pref = GTScriptLauncher_Pref();
			menu(self.menuName, edit=True, deleteAllItems=True);
			menuItem(parent=self.menuName, label="Config", divider=True);
			menuItem(parent=self.menuName, label="Menu Settings", command=pref.ShowDialog);
			if None != self.lastExecuted:
				menuItem(parent=self.menuName, label="Repeat", divider=True);
				menuItem(parent=self.menuName, label='Repeat: '+self.lastExecuted.name, echoCommand=True, command=self.lastExecuted.func);				
			menuItem(parent=self.menuName, label=" ", divider=True);
			self.AddMenuItems(self.menuName, self.scriptDir);

	def AddMenuItems(self, parentMenu, path):
		lDirs = []
		lFiles = [];
		for f in os.listdir(path):
			fullPath = os.path.join(path, f);
			if os.path.isdir(fullPath):
				lDirs.append((f, fullPath));
			else:
				base, ext = os.path.splitext(f);
				if ".py" == ext or ".mel" == ext:
					lFiles.append((f, ext, fullPath));
			
		cmd = GTScriptLauncher_Cmd('dir', path);
		menuItem(parent=parentMenu, label='Open "'+cmd.name+'" Folder', command=cmd.func);

		if len(lDirs) > 0:
			menuItem(parent=parentMenu, label="Folders", divider=True);
			for f in lDirs:
				subMenu = menuItem(parent=parentMenu, label=f[0], subMenu=True, tearOff=False);
				self.AddMenuItems(subMenu, f[1]);					

		if len(lFiles) > 0:
			menuItem(parent=parentMenu, label="Scripts", divider=True);
			for f in lFiles:
				cmd = GTScriptLauncher_Cmd(f[1], f[2]);
				menuItem(parent=parentMenu, label=f[0], echoCommand=True, command=cmd.func);
			
	def ExistsMenu(self):
		lMenus = window(self.mayaWindow, query=True, menuArray=True);
		bExist = (self.menuName in lMenus);
		return bExist;

def initializePlugin(mObject):
	mPlugin = OpenMayaMPx.MFnPlugin(mObject);
	try:
		GTScriptLauncher();
	except:
		sys.stderr.write('Failed load plugin : %s' % kPluginName);
		raise

def uninitializePlugin(mObject):
	mPlugin = OpenMayaMPx.MFnPlugin(mObject);
	try:
		deleteUI(kPluginName, menu=True);
	except:
		sys.stderr.write('Failed unload plugin : %s' % kPluginName);
		raise;
